/* -*- tab-width: 4 -*- */

/* This include file declares all symbols exported by the interpreter
 * kernel that may be used by applications or extensions.
 */

C_LINKAGE_BEGIN

/* Autoloading
 */
extern ELKObject P_Autoload P_((ELKObject, ELKObject));

/* Bignums
 */
extern ELKObject Make_Uninitialized_Bignum P_((int));
extern void Bignum_Normalize_In_Place P_((struct S_Bignum *));

/* Boolean operators
 */
extern ELKObject P_Booleanp P_((ELKObject));
extern ELKObject P_Not P_((ELKObject));
extern ELKObject P_Eq P_((ELKObject, ELKObject));
extern ELKObject P_Eqv P_((ELKObject, ELKObject));
extern ELKObject P_Equal P_((ELKObject, ELKObject));
extern ELKObject P_Empty_List_Is_False P_((ELKObject));
extern int Eqv P_((ELKObject, ELKObject));
extern int Equal P_((ELKObject, ELKObject));

/* Characters
 */
extern ELKObject Make_Char P_((int));
extern ELKObject P_Charp P_((ELKObject));
extern ELKObject P_Char_Upcase P_((ELKObject));
extern ELKObject P_Char_Downcase P_((ELKObject));
extern ELKObject P_Char_Eq P_((ELKObject, ELKObject));
extern ELKObject P_Char_Less P_((ELKObject, ELKObject));
extern ELKObject P_Char_Greater P_((ELKObject, ELKObject));
extern ELKObject P_Char_Eq_Less P_((ELKObject, ELKObject));
extern ELKObject P_Char_Eq_Greater P_((ELKObject, ELKObject));
extern ELKObject P_Char_CI_Eq P_((ELKObject, ELKObject));
extern ELKObject P_Char_CI_Less P_((ELKObject, ELKObject));
extern ELKObject P_Char_CI_Greater P_((ELKObject, ELKObject));
extern ELKObject P_Char_CI_Eq_Less P_((ELKObject, ELKObject));
extern ELKObject P_Char_CI_Eq_Greater P_((ELKObject, ELKObject));
extern ELKObject P_Char_Upper_Casep P_((ELKObject));
extern ELKObject P_Char_Lower_Casep P_((ELKObject));
extern ELKObject P_Char_Alphabeticp P_((ELKObject));
extern ELKObject P_Char_Numericp P_((ELKObject));
extern ELKObject P_Char_Whitespacep P_((ELKObject));
extern ELKObject P_Char_To_Integer P_((ELKObject));
extern ELKObject P_Integer_To_Char P_((ELKObject));

/* Continuations
 */
extern ELKObject P_Call_With_Current_Continuation P_((ELKObject));
extern ELKObject P_Dynamic_Wind P_((ELKObject, ELKObject, ELKObject));
extern ELKObject P_Control_Pointp P_((ELKObject));
extern ELKObject P_Control_Point_Environment P_((ELKObject));

/* Scheme strings --> C Strings
 */
extern CHAR *Get_String P_((ELKObject));
extern CHAR *Get_Strsym P_((ELKObject));

/* Debugging
 */
extern ELKObject P_Backtrace_List P_((int, ELKObject*));

/* Dump
 */
extern ELKObject P_Dump P_((ELKObject));

/* Lexical bindings, environments
 */
extern ELKObject P_The_Environment P_((void));
extern ELKObject P_Define P_((ELKObject));
extern ELKObject P_Set P_((ELKObject));
extern ELKObject P_Environment_To_List P_((ELKObject));
extern ELKObject P_Define_Macro P_((ELKObject));
extern ELKObject P_Boundp P_((ELKObject));
extern ELKObject P_Global_Environment P_((void));
extern ELKObject P_Environmentp P_((ELKObject));
extern ELKObject The_Environment, Global_Environment;

/* Error handling
 */
extern void Primitive_Error P_((CHAR *, ...));
extern void Fatal_Error P_((CHAR *, ...));
extern void Range_Error P_((ELKObject));
extern void Panic P_((const CHAR*));
extern ELKObject P_Error P_((int, ELKObject*));
extern ELKObject P_Reset P_((void));
extern const CHAR *Error_Tag;	/* will be removed in the near future */
extern const CHAR *Get_Error_Tag P_((void));
extern void Set_Error_Tag P_((const CHAR *));
extern void Set_App_Name P_((CHAR *));

/* Interrupts
 */
extern ELKObject P_Disable_Interrupts P_((void));
extern ELKObject P_Enable_Interrupts P_((void));
extern void Signal_Exit P_((int));

/* Features
 */
extern ELKObject P_Features P_((void));
extern ELKObject P_Featurep P_((ELKObject));
extern ELKObject P_Provide P_((ELKObject));
extern ELKObject P_Require P_((int, ELKObject*));

/* Memory allocation, garbage collection
 */
extern int GC_Debug;
extern ELKObject Alloc_Object P_((int, int, int));
extern void Register_Before_GC P_((void (*)(void)));
extern void Register_After_GC P_((void (*)(void)));
extern ELKObject P_Collect P_((void));
extern ELKObject P_Garbage_Collect_Status P_((int, ELKObject *));
#ifdef GENERATIONAL_GC
	extern ELKObject P_Collect_Incremental P_((void));
#endif

/* Files and ports
 */
extern ELKObject Curr_Input_Port, Curr_Output_Port;
extern ELKObject Standard_Input_Port, Standard_Output_Port;
extern void Reset_IO P_((int));
extern ELKObject P_Current_Input_Port P_((void));
extern ELKObject P_Current_Output_Port P_((void));
extern ELKObject P_Input_Portp P_((ELKObject));
extern ELKObject P_Output_Portp P_((ELKObject));
extern ELKObject P_Open_Input_File P_((ELKObject));
extern ELKObject P_Open_Output_File P_((ELKObject));
extern ELKObject P_Open_Input_Output_File P_((ELKObject));
extern ELKObject P_Eof_Objectp P_((ELKObject));
extern ELKObject P_With_Input_From_File P_((ELKObject, ELKObject));
extern ELKObject P_With_Output_To_File P_((ELKObject, ELKObject));
extern ELKObject P_Call_With_Input_File P_((ELKObject, ELKObject));
extern ELKObject P_Call_With_Output_File P_((ELKObject, ELKObject));
extern ELKObject P_Open_Input_String P_((ELKObject));
extern ELKObject P_Open_Output_String P_((void));
extern ELKObject P_Port_File_Name P_((ELKObject));
extern ELKObject P_Tilde_Expand P_((ELKObject));
extern ELKObject P_File_Existsp P_((ELKObject));
extern ELKObject P_Close_Input_Port P_((ELKObject));
extern ELKObject P_Close_Output_Port P_((ELKObject));
extern ELKObject P_Port_Line_Number P_((ELKObject));
extern ELKObject Terminate_File P_((ELKObject));
extern ELKObject Make_Port P_((int, FILE*, ELKObject));
extern int Path_Max P_((void));

/* Loading of files
 */
extern ELKObject P_Load P_((int, ELKObject*));
extern void Load_Source_Port P_((ELKObject));
extern void Load_File P_((CHAR *));

/* Pairs and lists
 */
extern ELKObject P_Cons P_((ELKObject, ELKObject));
extern ELKObject P_Car P_((ELKObject));
extern ELKObject P_Cdr P_((ELKObject));
extern ELKObject P_Set_Car P_((ELKObject, ELKObject));
extern ELKObject P_Set_Cdr P_((ELKObject, ELKObject));
extern ELKObject P_Listp P_((ELKObject));
extern ELKObject P_List P_((int, ELKObject*));
extern ELKObject P_Length P_((ELKObject));
extern ELKObject P_Nullp P_((ELKObject));
extern ELKObject P_Pairp P_((ELKObject));
extern ELKObject P_Cxr P_((ELKObject, ELKObject));
extern ELKObject P_Cddr P_((ELKObject));
extern ELKObject P_Cdar P_((ELKObject));
extern ELKObject P_Cadr P_((ELKObject));
extern ELKObject P_Caar P_((ELKObject));
extern ELKObject P_Cdddr P_((ELKObject));
extern ELKObject P_Cddar P_((ELKObject));
extern ELKObject P_Cdadr P_((ELKObject));
extern ELKObject P_Cdaar P_((ELKObject));
extern ELKObject P_Caddr P_((ELKObject));
extern ELKObject P_Cadar P_((ELKObject));
extern ELKObject P_Caadr P_((ELKObject));
extern ELKObject P_Caaar P_((ELKObject));
extern ELKObject P_Caaaar P_((ELKObject));
extern ELKObject P_Caaadr P_((ELKObject));
extern ELKObject P_Caadar P_((ELKObject));
extern ELKObject P_Caaddr P_((ELKObject));
extern ELKObject P_Cadaar P_((ELKObject));
extern ELKObject P_Cadadr P_((ELKObject));
extern ELKObject P_Caddar P_((ELKObject));
extern ELKObject P_Cadddr P_((ELKObject));
extern ELKObject P_Cdaaar P_((ELKObject));
extern ELKObject P_Cdaadr P_((ELKObject));
extern ELKObject P_Cdadar P_((ELKObject));
extern ELKObject P_Cdaddr P_((ELKObject));
extern ELKObject P_Cddaar P_((ELKObject));
extern ELKObject P_Cddadr P_((ELKObject));
extern ELKObject P_Cdddar P_((ELKObject));
extern ELKObject P_Cddddr P_((ELKObject));
extern ELKObject P_Append P_((int, ELKObject*));
extern ELKObject P_Append_Set P_((int, ELKObject*));
extern ELKObject P_Last_Pair P_((ELKObject));
extern ELKObject P_Reverse P_((ELKObject));
extern ELKObject P_Reverse_Set P_((ELKObject));
extern ELKObject P_List_Tail P_((ELKObject, ELKObject));
extern ELKObject P_List_Ref P_((ELKObject, ELKObject));
extern ELKObject P_Assq P_((ELKObject, ELKObject));
extern ELKObject P_Assv P_((ELKObject, ELKObject));
extern ELKObject P_Assoc P_((ELKObject, ELKObject));
extern ELKObject P_Memq P_((ELKObject, ELKObject));
extern ELKObject P_Memv P_((ELKObject, ELKObject));
extern ELKObject P_Member P_((ELKObject, ELKObject));
extern ELKObject P_Make_List P_((ELKObject, ELKObject));
extern ELKObject Copy_List P_((ELKObject));
extern int Fast_Length P_((ELKObject));
extern ELKObject Const_Cons P_((ELKObject, ELKObject));

/* Startup and termination
 */
extern ELKObject P_Command_Line_Args P_((void));
extern ELKObject P_Exit P_((int, ELKObject*));
extern void Elk_Init P_((int, CHAR **av, int, CHAR *));

/* malloc() and realloc()
 */
extern CHAR *Safe_Malloc P_((unsigned));
extern CHAR *Safe_Realloc P_((CHAR*, unsigned));

/* Numbers
 */
extern ELKObject Make_Integer P_((int));
extern ELKObject Make_Unsigned P_((unsigned));
extern ELKObject Make_Long P_((long));
extern ELKObject Make_Unsigned_Long P_((unsigned long));
extern ELKObject Make_Reduced_Flonum P_((double));
extern ELKObject Make_Flonum P_((double));
extern ELKObject P_Numberp P_((ELKObject));
extern ELKObject P_Complexp P_((ELKObject));
extern ELKObject P_Realp P_((ELKObject));
extern ELKObject P_Rationalp P_((ELKObject));
extern ELKObject P_Integerp P_((ELKObject));
extern ELKObject P_Abs P_((ELKObject));
extern ELKObject P_Zerop P_((ELKObject));
extern ELKObject P_Positivep P_((ELKObject));
extern ELKObject P_Negativep P_((ELKObject));
extern ELKObject P_Oddp P_((ELKObject));
extern ELKObject P_Evenp P_((ELKObject));
extern ELKObject P_Exactp P_((ELKObject));
extern ELKObject P_Inexactp P_((ELKObject));
extern ELKObject P_Exact_To_Inexact P_((ELKObject));
extern ELKObject P_Inexact_To_Exact P_((ELKObject));
extern ELKObject P_Inc P_((ELKObject));
extern ELKObject P_Dec P_((ELKObject));
extern int Generic_Equal P_((ELKObject, ELKObject));
extern int Generic_Less P_((ELKObject, ELKObject));
extern int Generic_Greater P_((ELKObject, ELKObject));
extern int Generic_Eq_Less P_((ELKObject, ELKObject));
extern int Generic_Eq_Greater P_((ELKObject, ELKObject));
extern ELKObject P_Generic_Equal P_((int, ELKObject*));
extern ELKObject P_Generic_Less P_((int, ELKObject*));
extern ELKObject P_Generic_Greater P_((int, ELKObject*));
extern ELKObject P_Generic_Eq_Less P_((int, ELKObject*));
extern ELKObject P_Generic_Eq_Greater P_((int, ELKObject*));
extern ELKObject P_Generic_Plus P_((int, ELKObject*));
extern ELKObject P_Generic_Minus P_((int, ELKObject*));
extern ELKObject P_Generic_Multiply P_((int, ELKObject*));
extern ELKObject P_Generic_Divide P_((int, ELKObject*));
extern ELKObject P_Quotient P_((ELKObject, ELKObject));
extern ELKObject P_Remainder P_((ELKObject, ELKObject));
extern ELKObject P_Modulo P_((ELKObject, ELKObject));
extern ELKObject P_Gcd P_((int, ELKObject*));
extern ELKObject P_Lcm P_((int, ELKObject*));
extern ELKObject P_Floor P_((ELKObject));
extern ELKObject P_Ceiling P_((ELKObject));
extern ELKObject P_Truncate P_((ELKObject));
extern ELKObject P_Round P_((ELKObject));
extern ELKObject P_Sqrt P_((ELKObject));
extern ELKObject P_Exp P_((ELKObject));
extern ELKObject P_Log P_((ELKObject));
extern ELKObject P_Sin P_((ELKObject));
extern ELKObject P_Cos P_((ELKObject));
extern ELKObject P_Tan P_((ELKObject));
extern ELKObject P_Asin P_((ELKObject));
extern ELKObject P_Acos P_((ELKObject));
extern ELKObject P_Atan P_((int, ELKObject*));
extern ELKObject P_Min P_((int, ELKObject*));
extern ELKObject P_Max P_((int, ELKObject*));
extern ELKObject P_Random P_((void));
extern ELKObject P_Srandom P_((ELKObject));
extern ELKObject P_Number_To_String P_((int, ELKObject*));
extern double Get_Double P_((ELKObject));
extern int Get_Integer P_((ELKObject));
extern unsigned Get_Unsigned P_((ELKObject));
extern long Get_Long P_((ELKObject));
extern unsigned long Get_Unsigned_Long P_((ELKObject));
extern int Get_Exact_Integer P_((ELKObject));
extern unsigned Get_Exact_Unsigned P_((ELKObject));
extern long Get_Exact_Long P_((ELKObject));
extern unsigned long Get_Exact_Unsigned_Long P_((ELKObject));

/* Onfork handlers
 */
extern void Register_Onfork P_((void (*)(void)));
extern void Call_Onfork P_((void));

/* Define_Primitive()
 */
extern void Define_Primitive P_((ELKObject (*)(ELLIPSIS), const CHAR*, int, int,
	enum discipline));

/* Output
 */
extern ELKObject P_Write P_((int, ELKObject*));
extern ELKObject P_Display P_((int, ELKObject*));
extern ELKObject P_Write_Char P_((int, ELKObject*));
extern ELKObject P_Newline P_((int, ELKObject*));
extern ELKObject P_Format P_((int, ELKObject*));
extern ELKObject P_Clear_Output_Port P_((int, ELKObject*));
extern ELKObject P_Flush_Output_Port P_((int, ELKObject*));
extern ELKObject P_Print P_((int, ELKObject*));
extern ELKObject P_Get_Output_String P_((ELKObject));
extern void Check_Output_Port P_((ELKObject));
extern void Discard_Output P_((ELKObject));
extern void Printf P_((ELKObject, CHAR *, ...));
extern void Print_Object P_((ELKObject, ELKObject, int, int, int));
extern void General_Print_Object P_((ELKObject, ELKObject, int));
extern void Format P_((ELKObject, const CHAR*, int, int, ELKObject*));
extern int Saved_Errno;

/* Evaluator, procedures, macros
 */
extern ELKObject Eval P_((ELKObject));
extern ELKObject P_Eval P_((int, ELKObject*));
extern ELKObject P_Apply P_((int, ELKObject*));
extern ELKObject Funcall P_((ELKObject, ELKObject, int));
extern ELKObject P_Lambda P_((ELKObject));
extern ELKObject P_Map P_((int, ELKObject*));
extern ELKObject P_Procedure_Environment P_((ELKObject));
extern ELKObject P_Procedure_Lambda P_((ELKObject));
extern ELKObject P_For_Each P_((int, ELKObject*));
extern ELKObject P_Procedurep P_((ELKObject));
extern ELKObject P_Macro P_((ELKObject));
extern ELKObject P_Macro_Body P_((ELKObject));
extern ELKObject P_Macro_Expand P_((ELKObject));
extern ELKObject P_Primitivep P_((ELKObject));
extern ELKObject P_Compoundp P_((ELKObject));
extern ELKObject P_Macrop P_((ELKObject));
extern void Check_Procedure P_((ELKObject));

/* Delay and force
 */
extern ELKObject P_Delay P_((ELKObject));
extern ELKObject P_Force P_((ELKObject));
extern ELKObject P_Promisep P_((ELKObject));
extern ELKObject P_Promise_Environment P_((ELKObject));

/* Input
 */
extern ELKObject P_Read P_((int, ELKObject*));
extern ELKObject P_Read_Char P_((int, ELKObject*));
extern ELKObject P_Peek_Char P_((int, ELKObject*));
extern ELKObject P_Char_Readyp P_((int, ELKObject*));
extern ELKObject P_Unread_Char P_((int, ELKObject*));
extern ELKObject P_Read_String P_((int, ELKObject*));
extern ELKObject P_Clear_Input_Port P_((int, ELKObject*));
extern ELKObject General_Read P_((ELKObject, int));
extern void Check_Input_Port P_((ELKObject));
extern void Discard_Input P_((ELKObject));
extern void Define_Reader P_((int, READFUN));

/* Special forms
 */
extern ELKObject P_Quote P_((ELKObject));
extern ELKObject P_If P_((ELKObject));
extern ELKObject P_Let P_((ELKObject));
extern ELKObject P_Letseq P_((ELKObject));
extern ELKObject P_Letrec P_((ELKObject));
extern ELKObject P_Case P_((ELKObject));
extern ELKObject P_Cond P_((ELKObject));
extern ELKObject P_And P_((ELKObject));
extern ELKObject P_Or P_((ELKObject));
extern ELKObject P_Do P_((ELKObject));
extern ELKObject P_Quasiquote P_((ELKObject));
extern ELKObject P_Fluid_Let P_((ELKObject));
extern ELKObject P_Begin P_((ELKObject));
extern ELKObject P_Begin1 P_((ELKObject));

/* Strings
 */
extern ELKObject Make_String P_((const CHAR*, int));
extern ELKObject Make_Const_String P_((const CHAR*, int));
extern ELKObject P_Make_String P_((int, ELKObject*));
extern ELKObject P_Stringp P_((ELKObject));
extern ELKObject P_String P_((int, ELKObject*));
extern ELKObject P_String_To_Number P_((int, ELKObject*));
extern ELKObject P_String_Eq P_((ELKObject, ELKObject));
extern ELKObject P_String_Less P_((ELKObject, ELKObject));
extern ELKObject P_String_Greater P_((ELKObject, ELKObject));
extern ELKObject P_String_Eq_Less P_((ELKObject, ELKObject));
extern ELKObject P_String_Eq_Greater P_((ELKObject, ELKObject));
extern ELKObject P_String_CI_Eq P_((ELKObject, ELKObject));
extern ELKObject P_String_CI_Less P_((ELKObject, ELKObject));
extern ELKObject P_String_CI_Greater P_((ELKObject, ELKObject));
extern ELKObject P_String_CI_Eq_Less P_((ELKObject, ELKObject));
extern ELKObject P_String_CI_Eq_Greater P_((ELKObject, ELKObject));
extern ELKObject P_String_Length P_((ELKObject));
extern ELKObject P_String_Ref P_((ELKObject, ELKObject));
extern ELKObject P_String_Set P_((ELKObject, ELKObject, ELKObject));
extern ELKObject P_Substring P_((ELKObject, ELKObject, ELKObject));
extern ELKObject P_String_Copy P_((ELKObject));
extern ELKObject P_String_Append P_((int, ELKObject*));
extern ELKObject P_List_To_String P_((ELKObject));
extern ELKObject P_String_To_List P_((ELKObject));
extern ELKObject P_Substring_Fill P_((ELKObject, ELKObject, ELKObject, ELKObject));
extern ELKObject P_String_Fill P_((ELKObject, ELKObject));
extern ELKObject P_Substringp P_((ELKObject, ELKObject));
extern ELKObject P_CI_Substringp P_((ELKObject, ELKObject));

/* Symbols, variables, frequently used Scheme objects
 */
extern ELKObject Null, Trueob, Falseob, Void, Newline, Eof;
extern ELKObject Intern P_((const CHAR*));
extern ELKObject CI_Intern P_((const CHAR*));
extern ELKObject P_Oblist P_((void));
extern ELKObject P_Symbolp P_((ELKObject));
extern ELKObject P_Symbol_To_String P_((ELKObject));
extern ELKObject P_String_To_Symbol P_((ELKObject));
extern ELKObject P_Put P_((int, ELKObject*));
extern ELKObject P_Get P_((ELKObject, ELKObject));
extern ELKObject P_Symbol_Plist P_((ELKObject));
extern void Define_Variable P_((ELKObject*, const CHAR*, ELKObject));
extern void Define_Symbol P_((ELKObject *, const CHAR*));
extern ELKObject Sym_Else;
extern ELKObject Var_Get P_((ELKObject));
extern void Var_Set P_((ELKObject, ELKObject));
extern int Var_Is_True P_((ELKObject));
extern unsigned long Symbols_To_Bits P_((ELKObject, int, SYMDESCR*));
extern ELKObject Bits_To_Symbols P_((unsigned long, int, SYMDESCR*));

/* Termination functions
 */
extern void Register_Object P_((ELKObject, GENERIC, PFO, int));
extern void Deregister_Object P_((ELKObject));
extern ELKObject Find_Object P_((int, GENERIC, MATCHFUN, ...));
extern void Terminate_Group P_((GENERIC));
extern void Terminate_Type P_((int));

/* Types, Define_Type()
 */
extern TYPEDESCR *Types;
extern ELKObject P_Type P_((ELKObject));
extern void Wrong_Type P_((ELKObject, int));
extern void Wrong_Type_Combination P_((ELKObject, const CHAR*));
extern int Define_Type P_((int, const CHAR*, int (*)(ELKObject), int,
	int (*)(ELKObject, ELKObject), int (*)(ELKObject, ELKObject),
	int (*)(ELKObject, ELKObject, int, int, int),
	int (*)(ELKObject*, void (*)(ELKObject*)) ));

/* Vectors
 */
extern ELKObject Make_Vector P_((int, ELKObject));
extern ELKObject Make_Const_Vector P_((int, ELKObject));
extern ELKObject P_Make_Vector P_((int, ELKObject*));
extern ELKObject P_Vectorp P_((ELKObject));
extern ELKObject P_Vector P_((int, ELKObject*));
extern ELKObject P_Vector_Length P_((ELKObject));
extern ELKObject P_Vector_Ref P_((ELKObject, ELKObject));
extern ELKObject P_Vector_Set P_((ELKObject, ELKObject, ELKObject));
extern ELKObject P_Vector_To_List P_((ELKObject));
extern ELKObject P_List_To_Vector P_((ELKObject));
extern ELKObject P_Vector_Fill P_((ELKObject, ELKObject));
extern ELKObject P_Vector_Copy P_((ELKObject));

C_LINKAGE_END
