/* -*- tab-width: 4 -*- */

/* Built-in primitives, Define_Primitive().
 */

#include "elkkernel.h"

struct Prim_Init {
	void *fun;
	CHAR *name;
	int minargs, maxargs;
	enum discipline disc;
} Primitives[] = {

	/* autoload.c: */
	{(void *)P_Autoload,              x_("autoload"),       2, 2,EVAL},

	/* bool.c: */
	{(void *)P_Booleanp,              x_("boolean?"),       1, 1,EVAL},
	{(void *)P_Not,                   x_("not"),            1, 1,EVAL},
	{(void *)P_Eq,                    x_("eq?"),            2, 2,EVAL},
	{(void *)P_Eqv,                   x_("eqv?"),           2, 2,EVAL},
	{(void *)P_Equal,                 x_("equal?"),         2, 2,EVAL},
	{(void *)P_Empty_List_Is_False,   x_("empty-list-is-false-for-backward-compatibility"), 1, 1,EVAL},

	/* char.c: */
	{(void *)P_Charp,                 x_("char?"),          1, 1,EVAL},
	{(void *)P_Char_To_Integer,       x_("char->integer"),  1, 1,EVAL},
	{(void *)P_Integer_To_Char,       x_("integer->char"),  1, 1,EVAL},
	{(void *)P_Char_Upper_Casep,      x_("char-upper-case?"),   1, 1,EVAL},
	{(void *)P_Char_Lower_Casep,      x_("char-lower-case?"),   1, 1,EVAL},
	{(void *)P_Char_Alphabeticp,      x_("char-alphabetic?"),   1, 1,EVAL},
	{(void *)P_Char_Numericp,         x_("char-numeric?"),  1, 1,EVAL},
	{(void *)P_Char_Whitespacep,      x_("char-whitespace?"),   1, 1,EVAL},
	{(void *)P_Char_Upcase,           x_("char-upcase"),    1, 1,EVAL},
	{(void *)P_Char_Downcase,         x_("char-downcase"),  1, 1,EVAL},
	{(void *)P_Char_Eq,               x_("char=?"),         2, 2,EVAL},
	{(void *)P_Char_Less,             x_("char<?"),         2, 2,EVAL},
	{(void *)P_Char_Greater,          x_("char>?"),         2, 2,EVAL},
	{(void *)P_Char_Eq_Less,          x_("char<=?"),        2, 2,EVAL},
	{(void *)P_Char_Eq_Greater,       x_("char>=?"),        2, 2,EVAL},
	{(void *)P_Char_CI_Eq,            x_("char-ci=?"),      2, 2,EVAL},
	{(void *)P_Char_CI_Less,          x_("char-ci<?"),      2, 2,EVAL},
	{(void *)P_Char_CI_Greater,       x_("char-ci>?"),      2, 2,EVAL},
	{(void *)P_Char_CI_Eq_Less,       x_("char-ci<=?"),     2, 2,EVAL},
	{(void *)P_Char_CI_Eq_Greater,    x_("char-ci>=?"),     2, 2,EVAL},

	/* cont.c: */
	{(void *)P_Control_Pointp,        x_("control-point?"), 1, 1,EVAL},
	{(void *)P_Call_With_Current_Continuation,
						                          x_("call-with-current-continuation"), 1, 1,EVAL},
	{(void *)P_Dynamic_Wind,          x_("dynamic-wind"),   3, 3,EVAL},
	{(void *)P_Control_Point_Environment,
						                          x_("control-point-environment"),  1, 1,EVAL},

	/* debug.c: */
	{(void *)P_Backtrace_List,        x_("backtrace-list"), 0, 1,VARARGS},

	/* dump.c: */
#ifdef CAN_DUMP
	{(void *)P_Dump,                  x_("dump"),           1, 1,EVAL},
#endif

	/* env.c: */
	{(void *)P_Environmentp,          x_("environment?"),   1, 1,EVAL},
	{(void *)P_The_Environment,       x_("the-environment"),0, 0,EVAL},
	{(void *)P_Global_Environment,    x_("global-environment"), 0, 0,EVAL},
	{(void *)P_Define,                x_("define"),         1, MANY, NOEVAL},
	{(void *)P_Define_Macro,          x_("define-macro"),   1, MANY, NOEVAL},
	{(void *)P_Set,                   x_("set!"),           2, 2,NOEVAL},
	{(void *)P_Environment_To_List,   x_("environment->list"),  1, 1,EVAL},
	{(void *)P_Boundp,                x_("bound?"),         1, 1,EVAL},

	/* error.c: */
	{(void *)P_Error,                 x_("error"),          2, MANY, VARARGS},
	{(void *)P_Reset,                 x_("reset"),          0, 0,EVAL},

	/* exception.c: */
	{(void *)P_Disable_Interrupts,    x_("disable-interrupts"), 0, 0,EVAL},
	{(void *)P_Enable_Interrupts,     x_("enable-interrupts"),  0, 0,EVAL},

	/* feature.c: */
	{(void *)P_Features,              x_("features"),       0, 0,EVAL},
	{(void *)P_Featurep,              x_("feature?"),       1, 1,EVAL},
	{(void *)P_Provide,               x_("provide"),        1, 1,EVAL},
	{(void *)P_Require,               x_("require"),        1, 3,VARARGS},

	/* heap.c: */
	{(void *)P_Collect,               x_("collect"),        0, 0,EVAL},
	{(void *)P_Garbage_Collect_Status,x_("garbage-collect-status"),0, 2,VARARGS},
#ifdef GENERATIONAL_GC
	{(void *)P_Collect_Incremental,   x_("collect-incremental"),  0, 0,EVAL},
#endif

	/* io.c: */
	{(void *)P_Port_File_Name,        x_("port-file-name"),        1, 1,EVAL},
	{(void *)P_Port_Line_Number,      x_("port-line-number"),      1, 1,EVAL},
	{(void *)P_Eof_Objectp,           x_("eof-object?"),           1, 1,EVAL},
	{(void *)P_Current_Input_Port,    x_("current-input-port"),    0, 0,EVAL},
	{(void *)P_Current_Output_Port,   x_("current-output-port"),   0, 0,EVAL},
	{(void *)P_Input_Portp,           x_("input-port?"),           1, 1,EVAL},
	{(void *)P_Output_Portp,          x_("output-port?"),          1, 1,EVAL},
	{(void *)P_Open_Input_File,       x_("open-input-file"),       1, 1,EVAL},
	{(void *)P_Open_Output_File,      x_("open-output-file"),      1, 1,EVAL},
	{(void *)P_Open_Input_Output_File,x_("open-input-output-file"),1, 1,EVAL},
	{(void *)P_Close_Input_Port,      x_("close-input-port"),      1, 1,EVAL},
	{(void *)P_Close_Output_Port,     x_("close-output-port"),     1, 1,EVAL},
	{(void *)P_With_Input_From_File,  x_("with-input-from-file"),  2, 2,EVAL},
	{(void *)P_With_Output_To_File,   x_("with-output-to-file"),   2, 2,EVAL},
	{(void *)P_Call_With_Input_File,  x_("call-with-input-file"),  2, 2,EVAL},
	{(void *)P_Call_With_Output_File, x_("call-with-output-file"), 2, 2,EVAL},
	{(void *)P_Open_Input_String,     x_("open-input-string"),     1, 1,EVAL},
	{(void *)P_Open_Output_String,    x_("open-output-string"),    0, 0,EVAL},
	{(void *)P_Tilde_Expand,          x_("tilde-expand"),          1, 1,EVAL},
	{(void *)P_File_Existsp,          x_("file-exists?"),          1, 1,EVAL},

	/* load.c: */
	{(void *)P_Load,                  x_("load"),           1, 2,VARARGS},

	/* list.c: */
	{(void *)P_Cons,                  x_("cons"),           2, 2,EVAL},
	{(void *)P_Car,                   x_("car"),            1, 1,EVAL},
	{(void *)P_Cdr,                   x_("cdr"),            1, 1,EVAL},
	{(void *)P_Caar,                  x_("caar"),           1, 1,EVAL},
	{(void *)P_Cadr,                  x_("cadr"),           1, 1,EVAL},
	{(void *)P_Cdar,                  x_("cdar"),           1, 1,EVAL},
	{(void *)P_Cddr,                  x_("cddr"),           1, 1,EVAL},

	{(void *)P_Caaar,                 x_("caaar"),          1, 1,EVAL},
	{(void *)P_Caadr,                 x_("caadr"),          1, 1,EVAL},
	{(void *)P_Cadar,                 x_("cadar"),          1, 1,EVAL},
	{(void *)P_Caddr,                 x_("caddr"),          1, 1,EVAL},
	{(void *)P_Cdaar,                 x_("cdaar"),          1, 1,EVAL},
	{(void *)P_Cdadr,                 x_("cdadr"),          1, 1,EVAL},
	{(void *)P_Cddar,                 x_("cddar"),          1, 1,EVAL},
	{(void *)P_Cdddr,                 x_("cdddr"),          1, 1,EVAL},

	{(void *)P_Caaaar,                x_("caaaar"),         1, 1,EVAL},
	{(void *)P_Caaadr,                x_("caaadr"),         1, 1,EVAL},
	{(void *)P_Caadar,                x_("caadar"),         1, 1,EVAL},
	{(void *)P_Caaddr,                x_("caaddr"),         1, 1,EVAL},
	{(void *)P_Cadaar,                x_("cadaar"),         1, 1,EVAL},
	{(void *)P_Cadadr,                x_("cadadr"),         1, 1,EVAL},
	{(void *)P_Caddar,                x_("caddar"),         1, 1,EVAL},
	{(void *)P_Cadddr,                x_("cadddr"),         1, 1,EVAL},
	{(void *)P_Cdaaar,                x_("cdaaar"),         1, 1,EVAL},
	{(void *)P_Cdaadr,                x_("cdaadr"),         1, 1,EVAL},
	{(void *)P_Cdadar,                x_("cdadar"),         1, 1,EVAL},
	{(void *)P_Cdaddr,                x_("cdaddr"),         1, 1,EVAL},
	{(void *)P_Cddaar,                x_("cddaar"),         1, 1,EVAL},
	{(void *)P_Cddadr,                x_("cddadr"),         1, 1,EVAL},
	{(void *)P_Cdddar,                x_("cdddar"),         1, 1,EVAL},
	{(void *)P_Cddddr,                x_("cddddr"),         1, 1,EVAL},

	{(void *)P_Cxr,                   x_("cxr"),            2, 2,EVAL},
	{(void *)P_Nullp, 	              x_("null?"),          1, 1,EVAL},
	{(void *)P_Pairp, 	              x_("pair?"),          1, 1,EVAL},
	{(void *)P_Listp,                 x_("list?"),          1, 1,EVAL},
	{(void *)P_Set_Car,               x_("set-car!"),       2, 2,EVAL},
	{(void *)P_Set_Cdr,               x_("set-cdr!"),       2, 2,EVAL},
	{(void *)P_Assq,                  x_("assq"),           2, 2,EVAL},
	{(void *)P_Assv,                  x_("assv"),           2, 2,EVAL},
	{(void *)P_Assoc,                 x_("assoc"),          2, 2,EVAL},
	{(void *)P_Memq,                  x_("memq"),           2, 2,EVAL},
	{(void *)P_Memv,                  x_("memv"),           2, 2,EVAL},
	{(void *)P_Member,                x_("member"),         2, 2,EVAL},
	{(void *)P_Make_List,             x_("make-list"),      2, 2,EVAL},
	{(void *)P_List,                  x_("list"),           0, MANY, VARARGS},
	{(void *)P_Length,                x_("length"),         1, 1,EVAL},
	{(void *)P_Append,                x_("append"),         0, MANY, VARARGS},
	{(void *)P_Append_Set,            x_("append!"),        0, MANY, VARARGS},
	{(void *)P_Last_Pair,             x_("last-pair"),      1, 1,EVAL},
	{(void *)P_Reverse,               x_("reverse"),        1, 1,EVAL},
	{(void *)P_Reverse_Set,           x_("reverse!"),       1, 1,EVAL},
	{(void *)P_List_Tail,             x_("list-tail"),      2, 2,EVAL},
	{(void *)P_List_Ref,              x_("list-ref"),       2, 2,EVAL},

	/* main.c: */
	{(void *)P_Command_Line_Args,     x_("command-line-args"),0, 0,EVAL},
	{(void *)P_Exit,                  x_("exit"),           0, 1,VARARGS},

	/* math.c: */
	{(void *)P_Number_To_String,      x_("number->string"), 1, 2,VARARGS},
	{(void *)P_Numberp,               x_("number?"),        1, 1,EVAL},
	{(void *)P_Complexp,              x_("complex?"),       1, 1,EVAL},
	{(void *)P_Realp,                 x_("real?"),          1, 1,EVAL},
	{(void *)P_Rationalp,             x_("rational?"),      1, 1,EVAL},
	{(void *)P_Integerp,              x_("integer?"),       1, 1,EVAL},
	{(void *)P_Zerop,                 x_("zero?"),          1, 1,EVAL},
	{(void *)P_Positivep,             x_("positive?"),      1, 1,EVAL},
	{(void *)P_Negativep,             x_("negative?"),      1, 1,EVAL},
	{(void *)P_Oddp,                  x_("odd?"),           1, 1,EVAL},
	{(void *)P_Evenp,                 x_("even?"),          1, 1,EVAL},
	{(void *)P_Exactp,                x_("exact?"),         1, 1,EVAL},
	{(void *)P_Inexactp,              x_("inexact?"),       1, 1,EVAL},
	{(void *)P_Exact_To_Inexact,      x_("exact->inexact"), 1, 1,EVAL},
	{(void *)P_Inexact_To_Exact,      x_("inexact->exact"), 1, 1,EVAL},
	{(void *)P_Generic_Less,          x_("<"),              1, MANY, VARARGS},
	{(void *)P_Generic_Greater,       x_(">"),              1, MANY, VARARGS},
	{(void *)P_Generic_Equal,         x_("="),              1, MANY, VARARGS},
	{(void *)P_Generic_Eq_Less,       x_("<="),             1, MANY, VARARGS},
	{(void *)P_Generic_Eq_Greater,    x_(">="),             1, MANY, VARARGS},
	{(void *)P_Inc,                   x_("1+"),             1, 1,EVAL},
	{(void *)P_Dec,                   x_("-1+"),            1, 1,EVAL},
	{(void *)P_Dec,                   x_("1-"),             1, 1,EVAL},
	{(void *)P_Generic_Plus,          x_("+"),              0, MANY, VARARGS},
	{(void *)P_Generic_Minus,         x_("-"),              1, MANY, VARARGS},
	{(void *)P_Generic_Multiply,      x_("*"),              0, MANY, VARARGS},
	{(void *)P_Generic_Divide,        x_("/"),              1, MANY, VARARGS},
	{(void *)P_Abs,                   x_("abs"),            1, 1,EVAL},
	{(void *)P_Quotient,              x_("quotient"),       2, 2,EVAL},
	{(void *)P_Remainder,             x_("remainder"),      2, 2,EVAL},
	{(void *)P_Modulo,                x_("modulo"),         2, 2,EVAL},
	{(void *)P_Gcd,                   x_("gcd"),            0, MANY, VARARGS},
	{(void *)P_Lcm,                   x_("lcm"),            0, MANY, VARARGS},
	{(void *)P_Floor,                 x_("floor"),          1, 1,EVAL},
	{(void *)P_Ceiling,               x_("ceiling"),        1, 1,EVAL},
	{(void *)P_Truncate,              x_("truncate"),       1, 1,EVAL},
	{(void *)P_Round,                 x_("round"),          1, 1,EVAL},
	{(void *)P_Sqrt,                  x_("sqrt"),           1, 1,EVAL},
	{(void *)P_Exp,                   x_("exp"),            1, 1,EVAL},
	{(void *)P_Log,                   x_("log"),            1, 1,EVAL},
	{(void *)P_Sin,                   x_("sin"),            1, 1,EVAL},
	{(void *)P_Cos,                   x_("cos"),            1, 1,EVAL},
	{(void *)P_Tan,                   x_("tan"),            1, 1,EVAL},
	{(void *)P_Asin,                  x_("asin"),           1, 1,EVAL},
	{(void *)P_Acos,                  x_("acos"),           1, 1,EVAL},
	{(void *)P_Atan,                  x_("atan"),           1, 2,VARARGS},
	{(void *)P_Min,                   x_("min"),            1, MANY, VARARGS},
	{(void *)P_Max,                   x_("max"),            1, MANY, VARARGS},
	{(void *)P_Random,                x_("random"),         0, 0,EVAL},
	{(void *)P_Srandom,               x_("srandom"),        1, 1,EVAL},

	/* prim.c: */

	/* print.c: */
	{(void *)P_Write,                 x_("write"),          1, 2,VARARGS},
	{(void *)P_Display,               x_("display"),        1, 2,VARARGS},
	{(void *)P_Write_Char,            x_("write-char"),     1, 2,VARARGS},
	{(void *)P_Newline,               x_("newline"),        0, 1,VARARGS},
	{(void *)P_Print,                 x_("print"),          1, 2,VARARGS},
	{(void *)P_Clear_Output_Port,     x_("clear-output-port"),0, 1,VARARGS},
	{(void *)P_Flush_Output_Port,     x_("flush-output-port"),0, 1,VARARGS},
	{(void *)P_Get_Output_String,     x_("get-output-string"),1, 1,EVAL},
	{(void *)P_Format,                x_("format"),         2, MANY, VARARGS},

	/* proc.c: */
	{(void *)P_Procedurep,            x_("procedure?"),     1, 1,EVAL},
	{(void *)P_Primitivep,            x_("primitive?"),     1, 1,EVAL},
	{(void *)P_Compoundp,             x_("compound?"),      1, 1,EVAL},
	{(void *)P_Macrop,                x_("macro?"),         1, 1,EVAL},
	{(void *)P_Eval,                  x_("eval"),           1, 2,VARARGS},
	{(void *)P_Apply,                 x_("apply"),          2, MANY, VARARGS},
	{(void *)P_Lambda,                x_("lambda"),         2, MANY, NOEVAL},
	{(void *)P_Procedure_Environment, x_("procedure-environment"),1, 1,EVAL},
	{(void *)P_Procedure_Lambda,      x_("procedure-lambda"),1, 1,EVAL},
	{(void *)P_Map,                   x_("map"),            2, MANY, VARARGS},
	{(void *)P_For_Each,              x_("for-each"),       2, MANY, VARARGS},
	{(void *)P_Macro,                 x_("macro"),          2, MANY, NOEVAL},
	{(void *)P_Macro_Body,            x_("macro-body"),     1, 1,EVAL},
	{(void *)P_Macro_Expand,          x_("macro-expand"),   1, 1,EVAL},

	/* promise.c: */
	{(void *)P_Delay,                 x_("delay"),          1, 1,NOEVAL},
	{(void *)P_Force,                 x_("force"),          1, 1,EVAL},
	{(void *)P_Promisep,              x_("promise?"),       1, 1,EVAL},
	{(void *)P_Promise_Environment,   x_("promise-environment"),1, 1,EVAL},

	/* read.c: */
	{(void *)P_Clear_Input_Port,      x_("clear-input-port"),0,1,VARARGS},
	{(void *)P_Read,                  x_("read"),           0, 1,VARARGS},
	{(void *)P_Read_Char,             x_("read-char"),      0, 1,VARARGS},
	{(void *)P_Read_String,           x_("read-string"),    0, 1,VARARGS},
	{(void *)P_Unread_Char,           x_("unread-char"),    1, 2,VARARGS},
	{(void *)P_Peek_Char,             x_("peek-char"),      0, 1,VARARGS},
	{(void *)P_Char_Readyp,           x_("char-ready?"),    0, 1,VARARGS},

	/* special.c: */
	{(void *)P_Quote,                 x_("quote"),          1, 1,NOEVAL},
	{(void *)P_Quasiquote,            x_("quasiquote"),     1, 1,NOEVAL},
	{(void *)P_Begin,                 x_("begin"),          1, MANY, NOEVAL},
	{(void *)P_Begin1,                x_("begin1"),         1, MANY, NOEVAL},
	{(void *)P_If,                    x_("if"),             2, MANY, NOEVAL},
	{(void *)P_Case,                  x_("case"),           2, MANY, NOEVAL},
	{(void *)P_Cond,                  x_("cond"),           1, MANY, NOEVAL},
	{(void *)P_Do,                    x_("do"),             2, MANY, NOEVAL},
	{(void *)P_Let,                   x_("let"),            2, MANY, NOEVAL},
	{(void *)P_Letseq,                x_("let*"),           2, MANY, NOEVAL},
	{(void *)P_Letrec,                x_("letrec"),         2, MANY, NOEVAL},
	{(void *)P_Fluid_Let,             x_("fluid-let"),      2, MANY, NOEVAL},
	{(void *)P_And,                   x_("and"),            0, MANY, NOEVAL},
	{(void *)P_Or,                    x_("or"),             0, MANY, NOEVAL},

	/* string.c: */
	{(void *)P_String,                x_("string"),         0, MANY, VARARGS},
	{(void *)P_Stringp,               x_("string?"),        1, 1,EVAL},
	{(void *)P_Make_String,           x_("make-string"),    1, 2,VARARGS},
	{(void *)P_String_Length,         x_("string-length"),  1, 1,EVAL},
	{(void *)P_String_To_Number,      x_("string->number"), 1, 2,VARARGS},
	{(void *)P_String_Ref,            x_("string-ref"),     2, 2,EVAL},
	{(void *)P_String_Set,            x_("string-set!"),    3, 3,EVAL},
	{(void *)P_Substring,             x_("substring"),      3, 3,EVAL},
	{(void *)P_String_Copy,           x_("string-copy"),    1, 1,EVAL},
	{(void *)P_String_Append,         x_("string-append"),  0, MANY, VARARGS},
	{(void *)P_List_To_String,        x_("list->string"),   1, 1,EVAL},
	{(void *)P_String_To_List,        x_("string->list"),   1, 1,EVAL},
	{(void *)P_String_Fill,           x_("string-fill!"),   2, 2,EVAL},
	{(void *)P_Substring_Fill,        x_("substring-fill!"),4, 4,EVAL},
	{(void *)P_String_Eq,             x_("string=?"),       2, 2,EVAL},
	{(void *)P_String_Less,           x_("string<?"),       2, 2,EVAL},
	{(void *)P_String_Greater,        x_("string>?"),       2, 2,EVAL},
	{(void *)P_String_Eq_Less,        x_("string<=?"),      2, 2,EVAL},
	{(void *)P_String_Eq_Greater,     x_("string>=?"),      2, 2,EVAL},
	{(void *)P_String_CI_Eq,          x_("string-ci=?"),    2, 2,EVAL},
	{(void *)P_String_CI_Less,        x_("string-ci<?"),    2, 2,EVAL},
	{(void *)P_String_CI_Greater,     x_("string-ci>?"),    2, 2,EVAL},
	{(void *)P_String_CI_Eq_Less,     x_("string-ci<=?"),   2, 2,EVAL},
	{(void *)P_String_CI_Eq_Greater,  x_("string-ci>=?"),   2, 2,EVAL},
	{(void *)P_Substringp,            x_("substring?"),     2, 2,EVAL},
	{(void *)P_CI_Substringp,         x_("substring-ci?"),  2, 2,EVAL},

	/* symbol.c: */
	{(void *)P_String_To_Symbol,      x_("string->symbol"), 1, 1,EVAL},
	{(void *)P_Oblist,                x_("oblist"),         0, 0,EVAL},
	{(void *)P_Symbolp,               x_("symbol?"),        1, 1,EVAL},
	{(void *)P_Symbol_To_String,      x_("symbol->string"), 1, 1,EVAL},
	{(void *)P_Put,                   x_("put"),            2, 3,VARARGS},
	{(void *)P_Get,                   x_("get"),            2, 2,EVAL},
	{(void *)P_Symbol_Plist,          x_("symbol-plist"),   1, 1,EVAL},

	/* type.c: */
	{(void *)P_Type,                  x_("type"),           1, 1,EVAL},

	/* vector.c: */
	{(void *)P_Vectorp,               x_("vector?"),        1, 1,EVAL},
	{(void *)P_Make_Vector,           x_("make-vector"),    1, 2,VARARGS},
	{(void *)P_Vector,                x_("vector"),         0, MANY, VARARGS},
	{(void *)P_Vector_Length,         x_("vector-length"),  1, 1,EVAL},
	{(void *)P_Vector_Ref,            x_("vector-ref"),     2, 2,EVAL},
	{(void *)P_Vector_Set,            x_("vector-set!"),    3, 3,EVAL},
	{(void *)P_Vector_To_List,        x_("vector->list"),   1, 1,EVAL},
	{(void *)P_List_To_Vector,        x_("list->vector"),   1, 1,EVAL},
	{(void *)P_Vector_Fill,           x_("vector-fill!"),   2, 2,EVAL},
	{(void *)P_Vector_Copy,           x_("vector-copy"),    1, 1,EVAL},

	{0}
};

/* The C-compiler can't initialize unions, thus the primitive procedures
 * must be created during run-time (the problem actually is that one can't
 * provide an intializer for the "tag" component of an S_Primitive).
 */

void Init_Prim (void) {
	register struct Prim_Init *p;
	ELKObject frame, prim, sym;

	for (frame = Car (The_Environment), p = Primitives; p->fun; p++) {
		prim = Make_Primitive ((ELKObject(*)(ELLIPSIS))p->fun, p->name, p->minargs, p->maxargs,
			p->disc);
		sym = Intern (p->name);
		frame = Add_Binding (frame, sym, prim);
	}
	Car (The_Environment) = frame;
	Memoize_Frame (frame);
}

void Define_Primitive (ELKObject (*fun)(ELLIPSIS), const CHAR *name, int min, int max, enum discipline disc) {
	ELKObject prim, sym, frame;
	GC_Node2;

	Set_Error_Tag (x_("define-primitive"));
	prim = Make_Primitive (fun, name, min, max, disc);
	sym = Null;
	GC_Link2 (prim, sym);
	sym = Intern (name);
	if (disc == EVAL && min != max)
		Primitive_Error (x_("~s: number of arguments must be fixed"), sym);
	frame = Add_Binding (Car (The_Environment), sym, prim);
	SYMBOL(sym)->value = prim;
	Car (The_Environment) = frame;
	GC_Unlink;
}
